Unit Main;

Interface

Uses
  Windows, Classes, SysUtils, DCPcrypt2, DCPhaval, DCPmd4, DCPmd5, DCPripemd128,
  DCPripemd160, DCPsha1, DCPsha256, DCPsha512, DCPtiger, RegExpr, ShellApi;

Type
  TResultSize = (rsO, rsKO, rsMO, rsGO);

Function GetParamVal(Const TaggedParm: String; Const IgnoreCase: Boolean = True): String;
Function ApplicationVersion(Const FileName: String; Const Separator: String = '.';
  Const N1: Boolean = True; Const N2: Boolean = True; Const N3: Boolean = True; Const N4: Boolean = True): String;
Function GetHaval(Const FileName: String): String;
Function GetMD4(Const FileName: String): String;
Function GetMD5(Const FileName: String): String;
Function GetRipeMD128(Const FileName: String): String;
Function GetRipeMD160(Const FileName: String): String;
Function GetSHA1(Const FileName: String): String;
Function GetSHA256(Const FileName: String): String;
Function GetSHA384(Const FileName: String): String;
Function GetSHA512(Const FileName: String): String;
Function GetTiger(Const FileName: String): String;
Function GetSize(Const FileName: String; Const ResultSize: TResultSize = rsO; Const SetRound: Boolean = False): String;
Function GetMatch(Const FileName, Mask: String; Const IsText: Boolean = False;
  Const Full: Boolean = False; Separator: String = ','): String;
Function ReplaceMatch(Const FileName: String; Mask, MaskReplace: String; Const IsText: Boolean = False;
  Const UseSubstitution: Boolean = False; Const Save: Boolean = False; Const LineBreak: String = '\r\n';
  Const Modifier: String = ''): String;
Function DeleteFileToRecycleBin(Const AFileName: String): Boolean;
Procedure ConsoleString(Const Chaine: String);
Procedure WriteHelp(Language: String);

Var
  switchPrefixes: TSysCharSet;
  Error:          Integer;

Implementation

Function GetParamVal(Const TaggedParm: String; Const IgnoreCase: Boolean = True): String;
Var
  I, Len:       Integer;
  Comp1, Comp2: String;
  Same:         Boolean;
Begin
  Result := '';
  Comp1  := TaggedParm + '=';
  //get length of parameter for copy and verf below
  Len    := Length(Comp1);

  //check all param for return value of param needed
  For i := 1 To ParamCount Do
  Begin
    //check prefix if valid
    If Not (ParamStr(I)[1] In switchPrefixes) Then
      Continue;

    //copy parameter name
    Comp2 := Copy(ParamStr(I), 2, Len);

    //check if same parameter name
    If IgnoreCase Then
      Same := AnsiSameText(Comp1, Comp2)
    Else
      Same := AnsiSameText(Comp1, Comp2);

    //return value and break for
    If Same Then
    Begin
      Result := Copy(ParamStr(I), Len + 2, Length(ParamStr(I)));
      break;
    End;
  End; // for
End;

Function ApplicationVersion(Const FileName: String; Const Separator: String = '.';
  Const N1: Boolean = True; Const N2: Boolean = True; Const N3: Boolean = True; Const N4: Boolean = True): String;
Var
  VerInfoSize, VerValueSize, Dummy: DWord;
  VerInfo:                          Pointer;
  VerValue:                         PVSFixedFileInfo;
Begin
  Result := '';

  VerInfoSize := GetFileVersionInfoSize(PChar(FileName), Dummy);
  {Deux solutions : }
  If VerInfoSize <> 0 Then
  {- Les info de version sont inclues }
  Begin
    {On alloue de la mmoire pour un pointeur sur les info de version : }
    GetMem(VerInfo, VerInfoSize);
    {On rcupre ces informations : }
    GetFileVersionInfo(PChar(FileName), 0, VerInfoSize, VerInfo);
    VerQueryValue(VerInfo, '\', Pointer(VerValue), VerValueSize);
    {On traite les informations ainsi rcupres : }
    With VerValue^ Do
    Begin
      If N1 Then
        Result := IntToStr(dwFileVersionMS Shr 16);
      If N2 Then
      Begin
        If Result <> '' Then
          Result := Result + Separator;
        Result := Result + IntToStr(dwFileVersionMS And $FFFF);
      End;
      If N3 Then
      Begin
        If Result <> '' Then
          Result := Result + Separator;
        Result := Result + IntToStr(dwFileVersionLS Shr 16);
      End;
      If N4 Then
      Begin
        If Result <> '' Then
          Result := Result + Separator;
        Result := Result + IntToStr(dwFileVersionLS And $FFFF);
      End;
    End;
    {On libre la place prcdemment alloue : }
    FreeMem(VerInfo, VerInfoSize);
  End
  Else
    {- Les infos de version ne sont pas inclues }
    {On dclenche une exception dans le programme : }
    Raise EAccessViolation.Create('version information not found');
End;

Function GetHash(Const FileName: String; Var Hash: TDCP_hash; Var Digest: Array Of Byte): String;
Var
  Source: TFileStream;
  I:      Integer;
Begin
  Source := NIL;
  Result := '';
  Try
    Source := TFileStream.Create(FileName, fmOpenRead);  // open the file specified by Edit1
  Except
    Raise EAccessViolation.Create('Unable to open file');
  End;
  If Source <> NIL Then
  Begin
    Try
    {$IFDEF DEBUG}
    if not Hash.SelfTest then
      Raise EAccessViolation.Create('Hash test failed something is wrong');
    {$ENDIF}
      Hash.Init;                                    // initialize it
      Hash.UpdateStream(Source, Source.Size);       // hash the stream contents
      Hash.Final(Digest);                           // produce the digest
    Finally
      Source.Free;
    End;
    For I := Low(Digest) To High(Digest) Do
      Result := Result + IntToHex(Digest[I], 2);
  End;
End;

Function GetHaval(Const FileName: String): String;
Var
  Hash:   TDCP_haval;
  Digest: Array[0..31] Of Byte;  // Haval produces a 256bit digest (32bytes)
Begin
  Result := '';
  Hash   := TDCP_haval.Create(NIL);
  Try
    Result := GetHash(FileName, TDCP_hash(Hash), Digest);
  Finally
    Hash.Free;
  End;
End;

Function GetMD4(Const FileName: String): String;
Var
  Hash:   TDCP_md4;
  Digest: Array[0..15] Of Byte;  // MD4 produces a 128bit digest (16bytes)
Begin
  Result := '';
  Hash   := TDCP_md4.Create(NIL);
  Try
    Result := GetHash(FileName, TDCP_hash(Hash), Digest);
  Finally
    Hash.Free;
  End;
End;

Function GetMD5(Const FileName: String): String;
Var
  Hash:   TDCP_md5;
  Digest: Array[0..15] Of Byte;  // MD5 produces a 128bit digest (16bytes)
Begin
  Result := '';
  Hash   := TDCP_md5.Create(NIL);
  Try
    Result := GetHash(FileName, TDCP_hash(Hash), Digest);
  Finally
    Hash.Free;
  End;
End;

Function GetRipeMD128(Const FileName: String): String;
Var
  Hash:   TDCP_ripemd128;
  Digest: Array[0..15] Of Byte;  // RipeMD-128 produces a 128bit digest (16bytes)
Begin
  Result := '';
  Hash   := TDCP_ripemd128.Create(NIL);
  Try
    Result := GetHash(FileName, TDCP_hash(Hash), Digest);
  Finally
    Hash.Free;
  End;
End;

Function GetRipeMD160(Const FileName: String): String;
Var
  Hash:   TDCP_ripemd160;
  Digest: Array[0..19] Of Byte;  // RipeMD-160 produces a 160bit digest (20bytes)
Begin
  Result := '';
  Hash   := TDCP_ripemd160.Create(NIL);
  Try
    Result := GetHash(FileName, TDCP_hash(Hash), Digest);
  Finally
    Hash.Free;
  End;
End;

Function GetSHA1(Const FileName: String): String;
Var
  Hash:   TDCP_sha1;
  Digest: Array[0..19] Of Byte;  // SHA1 produces a 160bit digest (20bytes)
Begin
  Result := '';
  Hash   := TDCP_sha1.Create(NIL);
  Try
    Result := GetHash(FileName, TDCP_hash(Hash), Digest);
  Finally
    Hash.Free;
  End;
End;

Function GetSHA256(Const FileName: String): String;
Var
  Hash:   TDCP_sha256;
  Digest: Array[0..31] Of Byte;  // SHA256 produces a 256bit digest (32bytes)
Begin
  Result := '';
  Hash   := TDCP_sha256.Create(NIL);
  Try
    Result := GetHash(FileName, TDCP_hash(Hash), Digest);
  Finally
    Hash.Free;
  End;
End;

Function GetSHA384(Const FileName: String): String;
Var
  Hash:   TDCP_sha384;
  Digest: Array[0..47] Of Byte;  // SHA384 produces a 384bit digest (48bytes)
Begin
  Result := '';
  Hash   := TDCP_sha384.Create(NIL);
  Try
    Result := GetHash(FileName, TDCP_hash(Hash), Digest);
  Finally
    Hash.Free;
  End;
End;

Function GetSHA512(Const FileName: String): String;
Var
  Hash:   TDCP_sha512;
  Digest: Array[0..63] Of Byte;  // SHA512 produces a 512bit digest (64bytes)
Begin
  Result := '';
  Hash   := TDCP_sha512.Create(NIL);
  Try
    Result := GetHash(FileName, TDCP_hash(Hash), Digest);
  Finally
    Hash.Free;
  End;
End;

Function GetTiger(Const FileName: String): String;
Var
  Hash:   TDCP_tiger;
  Digest: Array[0..23] Of Byte;  // Tiger produces a 192bit digest (24bytes)
Begin
  Result := '';
  Hash   := TDCP_tiger.Create(NIL);
  Try
    Result := GetHash(FileName, TDCP_hash(Hash), Digest);
  Finally
    Hash.Free;
  End;
End;

Function FileSize(Const fileName: WideString): Int64;
Var
  sr: TSearchRec;
Begin
  If FindFirst(fileName, faAnyFile, sr) = 0 Then
    Result := Int64(sr.FindData.nFileSizeHigh) Shl Int64(32) + Int64(sr.FindData.nFileSizeLow)
  Else
    Result := -1;
  FindClose(sr);
End;

Function GetSize(Const FileName: String; Const ResultSize: TResultSize = rsO; Const SetRound: Boolean = False): String;
Const
  B  = 1;         //byte
  KB = 1024 * B;  //kilobyte
  MB = 1024 * KB; //megabyte
  GB = 1024 * MB; //gigabyte
Var
  Size: Integer;
Begin
  Result := '';
  Size   := FileSize(FileName);
  If ResultSize = rsKO Then
  Begin
    If SetRound Then
      Result := IntToStr(Round(Size / KB))
    Else
      Result := FormatFloat('#.##', Size / KB);
  End
  Else
  If ResultSize = rsMO Then
  Begin
    If SetRound Then
      Result := IntToStr(Round(Size / MB))
    Else
      Result := FormatFloat('#.##', Size / MB);
  End
  Else
  If ResultSize = rsGO Then
  Begin
    If SetRound Then
      Result := IntToStr(Round(Size / GB))
    Else
      Result := FormatFloat('#.##', Size / GB);
  End
  Else
    Result := IntToStr(Size);
  If Result = '' Then
    Result := '0';
  If Copy(Result, 1, 1) = ',' Then
    Result := '0' + Result;
End;

Function GetMatch(Const FileName, Mask: String; Const IsText: Boolean = False;
  Const Full: Boolean = False; Separator: String = ','): String;
Var
  Txt: TStringList;
  r:   TRegExpr;
Begin
  Result := '';
  Txt    := TStringList.Create;
  r      := TRegExpr.Create;
  Try
    Try
    //set txt to search
      If Not IsText Then
        Txt.LoadFromFile(FileName)
      Else
        Txt.Text := FileName;

      r.Expression := Mask;
      If r.Exec(Txt.Text) Then
      Begin
        If Not Full Then
          Result := r.Match[0]
        Else
          Repeat
            If Result <> '' Then
              Result := Result + Separator;
            Result := Result + r.Match[0];
          Until Not r.ExecNext;
      End;
    Except
      on E: Exception Do
      Begin
        Error := 2;
        ConsoleString('Exception class: ' + E.ClassName + ' with message: ' + E.Message);
        {$IFDEF DEBUG}
        Readln;
        {$ENDIF}
      End;
    End;
  Finally
    Txt.Free;
    r.Free;
  End;
End;

Function ReplaceMatch(Const FileName: String; Mask, MaskReplace: String; Const IsText: Boolean = False;
  Const UseSubstitution: Boolean = False; Const Save: Boolean = False; Const LineBreak: String = '\r\n';
  Const Modifier: String = ''): String;
Var
  Txt: TStringList;
  r:   TRegExpr;
Begin
  Result := '';
  Txt    := TStringList.Create;
  r      := TRegExpr.Create;
  Try
    Try
      If LineBreak = '\r' Then
      Begin
        Txt.LineBreak         := #13;
        r.LineSeparators      := #13;
        r.LinePairedSeparator := '';
      End
      Else
      If LineBreak = '\n' Then
      Begin
        Txt.LineBreak         := #10;
        r.LineSeparators      := #10;
        r.LinePairedSeparator := '';
      End
      Else
      Begin
        Txt.LineBreak         := #13#10;
        r.LineSeparators      := #13#10;
        r.LinePairedSeparator := #13#10;
      End;

      r.ModifierStr := Modifier;
      //set txt to search
      If Not IsText Then
        Txt.LoadFromFile(FileName)
      Else
        Txt.Text := FileName;

      Mask         := StringReplace(Mask, '=quote=', '"', [rfReplaceAll, rfIgnoreCase]);
      MaskReplace  := StringReplace(MaskReplace, '=quote=', '"', [rfReplaceAll, rfIgnoreCase]);
      r.Expression := Mask;
      Txt.Text     := r.Replace(Txt.Text, MaskReplace, UseSubstitution);
      Result       := Txt.Text;

      If Not IsText And Save Then
        Txt.SaveToFile(FileName);
    Except
      on E: Exception Do
      Begin
        Error := 2;
        ConsoleString('Exception class: ' + E.ClassName + ' with message: ' + E.Message);
        {$IFDEF DEBUG}
        Readln;
        {$ENDIF}
      End;
    End;
  Finally
    Txt.Free;
    r.Free;
  End;
End;

Function DeleteFileToRecycleBin(Const AFileName: String): Boolean;
Var
  fos: TSHFileOpStruct;
Begin
  FillChar(fos, SizeOf(fos), 0);
  With fos Do
  Begin
    wFunc  := FO_DELETE;
    pFrom  := PChar(AFileName + #0);
    fFlags := FOF_ALLOWUNDO Or FOF_NOCONFIRMATION Or FOF_SILENT;
  End;
  Result := (0 = ShFileOperation(fos));
End;

Procedure ConsoleString(Const Chaine: String);
Var
  Result: String;
Begin
  SetLength(Result, Length(Chaine));
  CharToOem(PChar(Chaine), PChar(Result));
  Write(Result);
End;

Procedure WriteHelp(Language: String);
Var
  FileName, Contents, Line: String;
  Filelng:                  TextFile;
Begin
  Contents := '';
  If Language = '' Then
    Language := 'en';
  FileName := IncludeTrailingPathDelimiter(ExtractFilePath(ParamStr(0))) +
    Language + '.' + ApplicationVersion(ParamStr(0)) + '.lng';

  //write file if not exists
  If Not FileExists(FileName) Then
  Begin
    AssignFile(Filelng, FileName);
    Rewrite(Filelng);
    Try
      WriteLn(Filelng, 'Exemple usage: %FILEPATH% /version %FILENAME%');
      WriteLn(Filelng, ' return full version info with . separator of %FILENAME% ex: 1.2.3.4');
      WriteLn(Filelng, 'Action switch needed can be one of this:');
      WriteLn(Filelng, '    /version for retrieve info version in ressource of a file');
      WriteLn(Filelng, '    Optionnal switchs for action /version:');
      WriteLn(Filelng, '        /language=value ex fr show help French (need /h or /help used too)');
      WriteLn(Filelng, '        /n1 return "Major" number');
      WriteLn(Filelng, '        /n2 return "Minor" number');
      WriteLn(Filelng, '        /n3 return "Release" number');
      WriteLn(Filelng, '        /n4 return "Build" number');
      WriteLn(Filelng, '        write order of /nx is always /n1 /n2 ... even if you set /n2 /n1');
      WriteLn(Filelng, '        /separator=char ex - return version number with - beetween number (if two, more or none /nx switch specified.');
      WriteLn(Filelng, '    /haval for retrieve 256bit Haval hash of file');
      WriteLn(Filelng, '    /md4 for retrieve 128bit MD4 hash of file');
      WriteLn(Filelng, '    /md5 for retrieve 128bit MD5 hash of file');
      WriteLn(Filelng, '    /ripemd128 for retrieve 128bit RipeMD-128 hash of file');
      WriteLn(Filelng, '    /ripemd160 for retrieve 160bit RipeMD-160 hash of file');
      WriteLn(Filelng, '    /sha1 for retrieve 160bit SHA1 hash of file');
      WriteLn(Filelng, '    /sha256 for retrieve 256bit SHA256 hash of file');
      WriteLn(Filelng, '    /sha384 for retrieve 384bit SHA384 hash of file');
      WriteLn(Filelng, '    /sha512 for retrieve 512bit SHA512 hash of file');
      WriteLn(Filelng, '    /tiger for retrieve 192bit Tiger hash of file');
      WriteLn(Filelng, '    /size for retrieve size in octes of a file');
      WriteLn(Filelng, '    Optionnal switchs for action /size:');
      WriteLn(Filelng, '        /ko for show size in ko xxx,xx');
      WriteLn(Filelng, '        /mo for show size in mo xxx,xx');
      WriteLn(Filelng, '        /go for show size in go xxx,xx');
      WriteLn(Filelng, '        /round for show size when use parameter /ko or /mo or /go without ,xx');
      WriteLn(Filelng, '    /del for delete a file or dir, send in recycle bin');
      WriteLn(Filelng, '    /match for retrieve part of text or file contents');
      WriteLn(Filelng, '    Switchs needed for action /match:');
      WriteLn(Filelng, '        /mask="value" mask to search ex: "(t[a-z]{1,2}t)" found test trt but not tRt');
      WriteLn(Filelng, '    Optionnal switchs for action /match:');
      WriteLn(Filelng, '        /text if last parameter is not a file but a text between ""');
      WriteLn(Filelng, '        /full return all match, default return only first match');
      WriteLn(Filelng, '        /separator=char when use /full this is write between each match ex: separator=,');
      WriteLn(Filelng, '    /replace for replace retrieve part of text or file contents');
      WriteLn(Filelng, '    Switchs needed for action /replace:');
      WriteLn(Filelng, '        /mask="value" mask to search ex: "(t[a-z]{1,2}t)" found test trt but not tRt');
      WriteLn(Filelng, '        /maskreplace="value" mask to replace ex: "[$1]" replace test by [test] trt by [trt]');
      WriteLn(Filelng, '    Optionnal switchs for action /replace:');
      WriteLn(Filelng, '        /text if last parameter is not a file but a text between ""');
      WriteLn(Filelng, '        /save if last parameter is a file that save new text in');
      WriteLn(Filelng, '        /usesubstitution if you use $0, $1, ... in maskreplace that replace it with found of mask');
      WriteLn(Filelng, '        /linebreak=value spcifie a new line break, can be \r or \n default \r\n');
      WriteLn(Filelng, '        /modifier=value specifie modifier for mask to use full actived imsgxr full disabled -i-m-s-g-x-r see modifier in regex.txt');
      WriteLn(Filelng, '    For /math and /replace, more explain on regex can be found in regex.txt in fileinfo.exe dir');
      WriteLn(Filelng, '    it''s like perl regex, you can see match.bat and replace.bat too');
    Finally
      CloseFile(Filelng);
    End;
  End;

  //read lng file
  AssignFile(Filelng, FileName);
  Reset(Filelng);
  Try
    While Not EOF(Filelng) Do
    Begin
      ReadLn(Filelng, Line);
      Contents := Contents + sLineBreak + Line;
    End;
  Finally
    CloseFile(Filelng);
  End;

  //show help
  Contents := StringReplace(Contents, '%FILEPATH%', ParamStr(0), [rfReplaceAll, rfIgnoreCase]);
  Contents := StringReplace(Contents, '%FILENAME%', ExtractFileName(ParamStr(0)), [rfReplaceAll, rfIgnoreCase]);
  WriteLn('');
  WriteLn('--- FileInfo by Ouiouioui - www.toutenvrac.org');
  WriteLn('--- Version: ' + ApplicationVersion(ParamStr(0)));
  ConsoleString(Contents);
End;

End.

